"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _types = require("@babel/types");

var _utils = require("./utils");

var DefaultRefLabel = "ref";
var DefaultRefFactory = "React.useState";
var DefaultIgnoreMemberExpr = true;

function reactLabelSugar(_, options) {
  var _options$refFactory = options.refFactory,
      refFactory = _options$refFactory === void 0 ? DefaultRefFactory : _options$refFactory,
      _options$refLabel = options.refLabel,
      refLabel = _options$refLabel === void 0 ? DefaultRefLabel : _options$refLabel,
      _options$ignoreMember = options.ignoreMemberExpr,
      ignoreMemberExpr = _options$ignoreMember === void 0 ? DefaultIgnoreMemberExpr : _options$ignoreMember;
  return {
    visitor: {
      LabeledStatement: function LabeledStatement(path, state) {
        var node = path.node;
        var label = node.label,
            body = node.body;
        if (label.name !== refLabel) return;

        if (!(0, _types.isExpressionStatement)(body)) {
          throw new Error("ref sugar must be an expression statement");
        }

        var expression = body.expression;

        if (!(0, _types.isAssignmentExpression)(expression)) {
          throw new Error("ref sugar expression must be an assignment");
        }

        var left = expression.left,
            right = expression.right;

        if (!(0, _types.isIdentifier)(left)) {
          throw new Error("ref sugar assignment left must be an identifier");
        }

        var refItem = {
          identify: left,
          modifier: path.scope.generateUidIdentifier("set".concat((0, _utils.capitalize)(left.name)))
        };
        path.replaceWith((0, _types.variableDeclaration)("const", [(0, _types.variableDeclarator)((0, _types.arrayPattern)([refItem.identify, refItem.modifier]), (0, _types.callExpression)((0, _types.identifier)(refFactory), [right]))]));
        if (!state.reactRefs) state.reactRefs = [];
        state.reactRefs.push(refItem);
      },
      AssignmentExpression: function AssignmentExpression(path, state) {
        if (!state.reactRefs) return;
        var node = path.node;
        var ref = getTargetRef(node.left, state.reactRefs);
        if (!ref) return;

        if ((0, _types.isMemberExpression)(node.left)) {
          !ignoreMemberExpr && handleMemberExpression(node.left, ref, path);
        } else {
          var arrowReturn = node.operator === "=" ? node.right : (0, _types.binaryExpression)(node.operator.slice(0, -1), ref.identify, node.right);
          path.replaceWith((0, _types.callExpression)(ref.modifier, [(0, _types.arrowFunctionExpression)([ref.identify], arrowReturn)]));
        }
      },
      UpdateExpression: function UpdateExpression(path, state) {
        if (!state.reactRefs) return;
        var node = path.node;
        var ref = getTargetRef(node.argument, state.reactRefs);
        if (!ref) return;

        if ((0, _types.isMemberExpression)(node.argument)) {
          !ignoreMemberExpr && handleMemberExpression(node.argument, ref, path);
        } else {
          path.replaceWith((0, _types.callExpression)(ref.modifier, [(0, _types.arrowFunctionExpression)([ref.identify], (0, _types.binaryExpression)(node.operator === "++" ? "+" : "-", ref.identify, (0, _types.numericLiteral)(1)))]));
        }
      }
    }
  };
}
/**
 * Get target value from "Identifier" or "MemberExpression"
 * eg: a = 1;          =>  a
 * eg: obj.value = 1;  =>  obj
 */


function getTargetRef(leftValue, refs) {
  if (!(0, _types.isIdentifier)(leftValue) && !(0, _types.isMemberExpression)(leftValue)) return undefined;

  if ((0, _types.isIdentifier)(leftValue)) {
    var target = leftValue.name;
    return refs.find(function (ref) {
      return ref.identify.name === target;
    });
  } else {
    return getTargetRef(leftValue.object, refs);
  }
}
/**
 * move member expression to setState callback;
 * eg: obj.value = 1;  =>  setState(obj => { obj.value = 1 });
 */


function handleMemberExpression(node, ref, path) {
  var _node$extra, _node$extra2;

  if ((_node$extra = node.extra) === null || _node$extra === void 0 ? void 0 : _node$extra.REACT_REF_HAS_SETTED) return;
  path.replaceWith((0, _types.callExpression)(ref.modifier, [(0, _types.arrowFunctionExpression)([ref.identify], (0, _types.blockStatement)([(0, _types.expressionStatement)(path.node)]))]));
  (_node$extra2 = node.extra) !== null && _node$extra2 !== void 0 ? _node$extra2 : node.extra = {};
  node.extra.REACT_REF_HAS_SETTED = true;
}

var _default = reactLabelSugar;
exports["default"] = _default;